/*
 * Peony-Qt's Library
 *
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

#ifndef SIDEBAR_H
#define SIDEBAR_H
#include <QTreeView>
#include <QPaintEvent>
#include <QStyledItemDelegate>
#include <QStyleOptionViewItem>
#include <QModelIndex>
#include <QPainter>
#include <QProxyStyle>
#include <QStyleOption>
#include <PeonySideBarProxyModel>
#include <QComboBox>
#include <QSize>

class FileDialogComboBox : public QComboBox
{
    Q_OBJECT
public:
    explicit FileDialogComboBox(QWidget *parent = nullptr);
    QSize minimumSizeHint() const;

Q_SIGNALS:
    void setStyleChanged(bool change) const;

private:
    bool m_styleChanged = false;
    QSize m_minSize;
};


class FileDialogSideBar : public QTreeView
{

    Q_OBJECT
public:
    explicit FileDialogSideBar(QWidget *parent = nullptr);
    void resizeEvent(QResizeEvent *e);
    void paintEvent(QPaintEvent *event);

    QSize sizeHint() const;

public Q_SLOTS:
    void menuRequest(const QPoint &pos);

Q_SIGNALS:
    void goToUriRequest(const QString &uri, bool addToHistory = true, bool forceUpdate = false);

};


class SideBarItemDelegate : public QStyledItemDelegate
{
    friend class FileDialogSideBar;
    explicit SideBarItemDelegate(QObject *parent = nullptr);

    QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const override;
    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const;
};


class SideBarStyle : public QProxyStyle
{
public:
    explicit SideBarStyle();
    void drawPrimitive(PrimitiveElement element, const QStyleOption *option, QPainter *painter, const QWidget *widget) const override;
    void drawControl(ControlElement element, const QStyleOption *option, QPainter *painter, const QWidget *widget) const override;
};
#endif // SIDEBAR_H
